# vi: ts=8 sw=8

package Tongues::Belaruski;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Cyrillic CP1251
# not KOI8-R or ISO 8859-5

# Alphabetical order
#                                
#                                

# Digraphs not considered separate letters for sorting
#   
#   

# Extra characters (ISO & Windows)
# '

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::CyrillicWin;

$charset = new Charsets::CyrillicWin;

# Table for converting words to root form
$grammar = [
 # Noun case
 # Noun gender
 # Noun plural
 # Adjectives
 # Verb past participle
 # Verb gerund
 # Verb present participle
 # Verb present
 # Verb present subjunctive
 # Verb imperitive
 # Verb preterite
 # Verb imperfect
 # Verb future
 # Verb conditional
 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translation		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine
#	n -> neuter
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> case:
#	n -> nominative 
#	g -> genitive 
#	d -> dative 
#	a -> accusative 
#	v -> vocative 
#	l -> locative 
#	inst -> instrumental 

# Russian to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 #     Plural
 #    Feminine
 #     Singular
 #     Plural
 #   Indefinite articles
 #  Pronouns & possessive adjectives
 #   1st person
 #   2nd person
 #   3rd person
 #  Other functional words
 #   Partitive / Contractions
 # Numbers: cardinal and ordinal
 # Days and months
 ''	=> { 'x' => 'sunday' },
 ''	=> { 'x' => 'monday' },
 ''	=> { 'x' => 'tuesday' },
 ''	=> { 'x' => 'wednesday' },
 ''	=> { 'x' => 'thursday' },
 ''	=> { 'x' => 'friday' },
 ''	=> { 'x' => 'saturday' },
 ''	=> { 'x' => 'january' },
 ''		=> { 'x' => 'february' },
 ''	=> { 'x' => 'march' },
 ''	=> { 'x' => 'april' },
 ''		=> { 'x' => 'may' },
 ''	=> { 'x' => 'june' },
 ''	=> { 'x' => 'july' },
 ''	=> { 'x' => 'august' },
 ''	=> { 'x' => 'september' },
 ''	=> { 'x' => 'october' },
 ''	=> { 'x' => 'november' },
 ''	=> { 'x' => 'december' },
 # Key verbs
 # Vocabulary
 ''  => { 'x' => 'article',
	         't' => 'n' },
);
}

1;

