# vi: ts=8 sw=8 sts=8

package Tongues::Francais;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

# Extra characters (ISO & Windows)
#            
#            
# Extra characters (Mac)
#            
#            

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun gender
 [ 'e',		'',	'',		'',	'n' ], # fem.
 # Noun plural
 [ 's',		'',	'',		's',	'n' ],
 [ 'aux',	'',	'',		's',	'n' ],
 [ 'x',		'',	'',		's',	'n' ],
 # Adjectives
 [ 's',		'',	'',		'',	'a' ],	# masc. pl.
 [ 'e',		'',	'',		'',	'a' ],	# fem.
 [ 'ieuse',	'ieux',	'',		'',	'a' ],	# fem.
 [ 'es',	'',	'',		'',	'a' ],	# fem. pl.
 # Verb past participle
 [ '',		'er',	'',		'ed',	'v' ],
 [ 'i',		'ir',	'',		'ed',	'v' ],
 [ 'u',		're',	'',		'ed',	'v' ],
 # Verb gerund
 # Verb present participle
 # Verb present (prsent)
 [ 'e',		'er',	'',	's',	'v' ],	# i, she, one, it
 [ 'es',	'er',	'',	'',	'v' ],
 [ 'ons',	'er',	'',	'',	'v' ],
 [ 'ez',	'er',	'',	'',	'v' ],
 [ 'ent',	'er',	'',	'',	'v' ],
 [ 'is',	'ir',	'',	'',	'v' ],
 [ 'it',	'ir',	'',	's',	'v' ],	# she, one, it
 [ 'issons',	'ir',	'',	'',	'v' ],
 [ 'issez',	'ir',	'',	'',	'v' ],
 [ 'issent',	'ir',	'',	'',	'v' ],
 [ 's',		're',	'',	'',	'v' ],
 [ '',		're',	'',	's',	'v' ],	# she, one, it
 [ 'ons',	're',	'',	'',	'v' ],
 [ 'ez',	're',	'',	'',	'v' ],
 [ 'ent',	're',	' ',	'',	'v' ],
 # Verb present subjunctive (subjonctif)
 # Verb imperitive (impratif)
 # Verb preterite (pass simple, literary)
 # Verb imperfect (imparfait)
 [ 'ais',	'er',	'',	'ed',	'v' ],
 [ 'ait',	'er',	'',	'ed',	'v' ],
 [ 'ions',	'er',	'',	'ed',	'v' ],
 [ 'iez',	'er',	'',	'ed',	'v' ],
 [ 'aient',	'er',	'',	'ed',	'v' ],
 [ 'issais',	'ir',	'',	'ed',	'v' ],
 [ 'issait',	'ir',	'',	'ed',	'v' ],
 [ 'issions',	'ir',	'',	'ed',	'v' ],
 [ 'issiez',	'ir',	'',	'ed',	'v' ],
 [ 'issaient',	'ir',	'',	'ed',	'v' ],
 [ 'ais',	're',	'',	'ed',	'v' ],
 [ 'ait',	're',	'',	'ed',	'v' ],
 [ 'ions',	're',	'',	'ed',	'v' ],
 [ 'iez',	're',	'',	'ed',	'v' ],
 [ 'aient',	're',	'',	'ed',	'v' ],
 # Verb future (futur)
 [ 'erai',	'er',	'will ',	'',	'v' ],
 [ 'eras',	'er',	'will ',	'',	'v' ],
 [ 'era',	'er',	'will ',	'',	'v' ],
 [ 'erons',	'er',	'will ',	'',	'v' ],
 [ 'erez',	'er',	'will ',	'',	'v' ],
 [ 'eront',	'er',	'will ',	'',	'v' ],
 [ 'rai',	're',	'will ',	'',	'v' ],
 [ 'ras',	're',	'will ',	'',	'v' ],
 [ 'ra',	're',	'will ',	'',	'v' ],
 [ 'rons',	're',	'will ',	'',	'v' ],
 [ 'rez',	're',	'will ',	'',	'v' ],
 [ 'ront',	're',	'will ',	'',	'v' ],
 # Verb conditional (conditionnel)
 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translation		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjunction
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default (TODO for words ending in age ier ion ment oir)
#	f -> feminine		default (TODO for words ending in ade aison tion (sion)
#				ance (ence) erie ise tude t (t)ure ale (ole) aix (oix) oire)
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
#   -> plural:				nouns
#	aux -> add +aux suffix
#	x -> add +x suffix
# c -> conjugation (verbs):
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> conjugation (pronouns):
#	s -> subject
#	do -> direct object
#	io -> indirect object

# French to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine Singular
 'le'		=> { 'x' => 'the',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 's' },
 #    Feminine Singular
 'la'		=> { 'x' => 'the',
 		     't' => 'art',
		     'g' => 'f',
		     'n' => 's' },
 #    Plural
 'les'		=> { 'x' => 'the',
		     '#' => 'masc., fem., plural; pron. = them' },
 #   Indefinite articles
 'un'		=> { 'x' => 'a',
		     '#' => 'an, one',
		     't' => 'art',
		     'g' => 'm' },
 'une'		=> { 'x' => 'a',
		     '#' => 'an, one',
		     't' => 'art',
		     'g' => 'f' },
 #  Pronouns & possessive adjectives
 #   1st person
 'je'		=> { 'x' => 'i',
		     '#' => 'subj',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'me'		=> { 'x' => 'me',
		     '#' => 'dir/indir obj',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'moi'		=> { 'x' => 'me',
		     '#' => '?',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'mon'		=> { 'x' => 'my',
		     'p' => '1',
		     'n' => 's' },
 'ma'		=> { 'x' => 'my',
		     'p' => '1',
		     'n' => 's' },
 'mes'		=> { 'x' => 'my',
		     'p' => '1',
		     'n' => 's' },
 'nous'		=> { 'x' => 'we',
		     '#' => 'us; subj/dir/indir obj',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p' },
 'notre'	=> { 'x' => 'our',
		     '#' => 'masc&fem sing.',
		     'p' => '1',
		     'n' => 'p' },
 'nos'		=> { 'x' => 'our',
		     '#' => 'plural',
		     'p' => '1',
		     'n' => 'p' },
 #   2nd person
 'tu'		=> { 'x' => 'you',
 		     't' => 'pro',
		     'p' => '2',
		     '#' => 'informal',
		     'n' => 's' },
 'te'		=> { 'x' => 'you',
		     '#' => 'dir/indir obj',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 's' },
 'ton'		=> { 'x' => 'your',
		     'p' => '2',
		     'n' => 's' },
 'ta'		=> { 'x' => 'your',
		     'p' => '2',
		     'n' => 's' },
 'tes'		=> { 'x' => 'your',
		     'p' => '2',
		     'n' => 's' },
 'vous'		=> { 'x' => 'you',
		     '#' => 'subj/dir/indir obj',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p' },
 'votre'	=> { 'x' => 'your',
		     '#' => 'masc&fem sing.',
		     'p' => '2',
		     'n' => 'p' },
 'vos'		=> { 'x' => 'your',
		     '#' => 'plural',
		     'p' => '2',
		     'n' => 'p' },
 #   3rd person
 'il'		=> { 'x' => 'he',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
# 'le'		=> { 'x' => 'him',
#		     '#' => 'dir obj',
# 		     't' => 'pro',
#		     'p' => '3',
#		     'n' => 's',
#		     'g' => 'm' },
 'lui'		=> { 'x' => 'it',
		     '#' => 'him/her; indir obj',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
 'son'		=> { 'x' => 'their',
		     '#' => 'really his/her/its',
		     'p' => '3',
		     'n' => 's' },
 'sa'		=> { 'x' => 'their',
		     '#' => 'really his/her/its',
		     'p' => '3',
		     'n' => 's' },
 'ses'		=> { 'x' => 'their',
		     '#' => 'really his/her/its',
		     'p' => '3',
		     'n' => 's' },
 'ils'		=> { 'x' => 'they',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'g' => 'm' },
# 'les'		=> { 'x' => 'them',
#		     '#' => 'dir obj',
# 		     't' => 'pro',
#		     'p' => '3',
#		     'n' => 'p' },
 'leur'		=> { 'x' => 'them',
		     '#' => 'indir obj; their - poss adj. 3p sing.',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p' },
 'leurs'	=> { 'x' => 'their',
		     '#' => 'poss adj. 3p plur.',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p' },
 'elle'		=> { 'x' => 'she',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'f' },
# 'la'		=> { 'x' => 'her',
#		     '#' => 'dir/indir obj',
# 		     't' => 'pro',
#		     'p' => '3',
#		     'n' => 's',
#		     'g' => 'f' },
 'elles'	=> { 'x' => 'they',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'g' => 'f' },
 #   Indefinite pronouns
 'on'		=> { 'x' => 'one',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
# 'en'		=> { 'x' => 'some',
# 		     't' => 'pro' },
 #  Interrogatives
 'o'		=> { 'x' => 'where',
		     '#' => 'interr.; cf. ou',
 		     't' => 'adv' },
 'quand'	=> { 'x' => 'when',
		     '#' => 'interr adv; conj.',
 		     't' => 'adv' },

 #  Other functional words
 ''		=> { 'x' => 'to',
		     '#' => 'at',
 		     't' => 'p' },
# 'au'		=> { 'x' => ' le' },
 'aussi'	=> { 'x' => 'as',
		     '#' => 'in comparisons' },
 'autre'	=> { 'x' => 'other' },
 'avec'		=> { 'x' => 'with',
 		     't' => 'p' },
 'a'		=> { 'x' => 'that' },
# 'c\''		=> { 'x' => 'ce' },
 'ce'		=> { 'x' => 'that',
		     '#' => 'this; those; these',
 		     't' => 'pro' },
  'cet'		=> { 'x' => 'that',
  		     'r' => 'ce' },
  'cette'	=> { 'x' => 'that',
  		     'r' => 'ce' },
  'ces'		=> { 'x' => 'those',
  		     'r' => 'ce' },
 'ceci'		=> { 'x' => 'this',
 		     't' => 'pro' },
 'cela'		=> { 'x' => 'that' },
 'chaque'	=> { 'x' => 'each',
		     '#' => 'every',
 		     't' => 'a' },
 'ci'		=> { 'x' => 'this',
 		     't' => 'adv' },
 'dans'		=> { 'x' => 'in',
		     't' => 'p' },
 'de'		=> { 'x' => 'of',
 		     't' => 'p' },
 'en'		=> { 'x' => 'in',
		     '#' => 'also "some"',
		     't' => 'p' },
 'et'		=> { 'x' => 'and',
 		     't' => 'conj' },
 'entre'	=> { 'x' => 'between',
		     't' => 'p' },
 'ici'		=> { 'x' => 'here',
 		     't' => 'adv' },
 'jamais'	=> { 'x' => 'ever',
		     '#' => 'ne jamais = never',
 		     't' => 'adv' },
 'l'		=> { 'x' => 'there',
		     '#' => 'cf "y"',
 		     't' => 'adv' },
 'mais'		=> { 'x' => 'but',
 		     't' => 'conj' },
 'mme'		=> { 'x' => 'same',
		     '#' => 'adj, pron, adv; even' },
 'moins'	=> { 'x' => 'less',
		     '#' => 'in comparisons' },
 'ne'		=> { 'x' => 'not',
		     '#' => 'cf. pas' },
 'ou'		=> { 'x' => 'or',
		     '#' => 'cf. o',
 		     't' => 'conj' },
 'par'		=> { 'x' => 'by',
		     '#' => 'through',
 		     'x-es' => 'por',
		     '#' => '?',
		     't' => 'p' },
 'parfois'	=> { 'x' => 'sometimes',
		     '#' => 'occasionally',
 		     't' => 'adv' },
 'pas'		=> { 'x' => 'not',
		     '#' => 'cf. ne' },
 'pendant'	=> { 'x' => 'during',
		     't' => 'p' },
 'plus'		=> { 'x' => 'more',
		     '#' => 'in comparisons; ne .. plus = not any more' },
 'pour'		=> { 'x' => 'for',
		     '#' => 'instead of, for the sake of, as, to',
 		     'x-es' => 'para',
		     '#' => '?',
 		     't' => 'p' },
 'pourtant'	=> { 'x' => 'yet',
 		     't' => 'adv' },
 'que'		=> { 'x' => 'that' },
 'quelque'	=> { 'x' => 'some',
		     '#' => 'adj.?' },
 'qui'		=> { 'x' => 'who',
		     '#' => 'which; interr. & relative pronoun',
 		     't' => 'pro' },
 'quois'	=> { 'x' => 'which' },
 'rien'		=> { 'x' => 'anything',
		     '#' => 'ne rien = nothing',
 		     't' => 'pro' },
 'sans'		=> { 'x' => 'without',
 		     't' => 'p' },
 'si'		=> { 'x' => 'if' },
 'sur'		=> { 'x' => 'on',
 		     't' => 'p' },
 'tellement'	=> { 'x' => 'so',
		     '#' => 'in such a way',
 		     't' => 'adv' },
 'trs'		=> { 'x' => 'very',
		     't' => 'adv' },
 'trop'		=> { 'x' => 'too',
		     '#' => 'demasiado, too much',
		     't' => 'adv' },
 'y'		=> { 'x' => 'there',
		     '#' => 'to it; cf "l"',
 		     't' => 'adv' },
 #   Partitive / Contractions
 'des'		=> { 'x' => 'of the' },
 'du'		=> { 'x' => 'of the' },
 # Numbers: cardinal and ordinal
 'zro'		=> { 'x' => 'zero' },
 # 'un'
 'premier'	=> { 'x' => 'first',
		     '#' => 'ordinal',
 		     't' => 'a' },
 'premire'	=> { 'x' => 'first',
		     '#' => 'ordinal',
 		     't' => 'a' },
 'deux'		=> { 'x' => 'two' },
 'second'	=> { 'x' => 'second',
		     '#' => 'ordinal',
 		     't' => 'a' },
 'deuxime'	=> { 'x' => 'second',
		     '#' => 'ordinal',
 		     't' => 'a' },
 'trois'	=> { 'x' => 'three' },
 'troisime'	=> { 'x' => 'third',
		     '#' => 'ordinal',
 		     't' => 'a' },
 'quatre'	=> { 'x' => 'four' },
 'cinq'		=> { 'x' => 'five' },
 'six'		=> { 'x' => 'six' },
 'sept'		=> { 'x' => 'seven' },
 'huit'		=> { 'x' => 'eight' },
 'neuf'		=> { 'x' => 'nine' },
 'dix'		=> { 'x' => 'ten' },
 'onze'		=> { 'x' => 'eleven' },
 'douze'	=> { 'x' => 'twelve' },
 'treize'	=> { 'x' => 'thirteen' },
 'quatorze'	=> { 'x' => 'fourteen' },
 'quinze'	=> { 'x' => 'fifteen' },
 'seize'	=> { 'x' => 'sixteen' },
 'vingt'	=> { 'x' => 'twenty' },
 'trente'	=> { 'x' => 'thirty' },
 'quarante'	=> { 'x' => 'forty' },
 'cinquante'	=> { 'x' => 'fifty' },
 'soixante'	=> { 'x' => 'sixty' },
 'cent'		=> { 'x' => 'hundred' },
 'mille'	=> { 'x' => 'thousand' },
 'million'	=> { 'x' => 'million' },

 # Days and months
 #  French days are not capitalized
 'dimanche'	=> { 'x' => 'sunday' },
 'lundi'	=> { 'x' => 'monday' },
 'mardi'	=> { 'x' => 'tuesday' },
 'mercredi'	=> { 'x' => 'wednesday' },
 'jeudi'	=> { 'x' => 'thursday' },
 'vendredi'	=> { 'x' => 'friday' },
 'samedi'	=> { 'x' => 'saturday' },
 #  French months are not capitalized
 'janvier'	=> { 'x' => 'january' },
 'fvrier'	=> { 'x' => 'february' },
 'mars'		=> { 'x' => 'march' },
 'avril'	=> { 'x' => 'april' },
 'mai'		=> { 'x' => 'may' },
 'juin'		=> { 'x' => 'june' },
 'juillet'	=> { 'x' => 'july' },
 'aot'		=> { 'x' => 'august' },
 'septembre'	=> { 'x' => 'september' },
 'octobre'	=> { 'x' => 'october' },
 'novembre'	=> { 'x' => 'november' },
 'dcembre'	=> { 'x' => 'december' },
 # Key Verbs
 #  tre - be
 'tre'		=> { 'x' => 'be',
		     't' => 'v' },
  'suis'	=> { 'x' => 'am',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'es'		=> { 'x' => 'are',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'p' },
  'est'		=> { 'x' => 'is',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'p' },
  'sommes'	=> { 'x' => 'are',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'p' },
  'tes'	=> { 'x' => 'are',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'p' },
  'sont'	=> { 'x' => 'are',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'p' },
  'tais'	=> { 'x' => 'was',
 		     'r' => 'tre',
		     't' => 'v',
#		     'p' => '1',
		     'n' => 's',
		     'c' => 'i' },
#  'tais'	=> { 'x' => 'were',
# 		     'r' => 'tre',
#		     't' => 'v',
#		     'p' => '2',
#		     'n' => 's',
#		     'c' => 'i' },
  'tait'	=> { 'x' => 'was',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'i' },
  'tions'	=> { 'x' => 'were',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'i' },
  'tiez'	=> { 'x' => 'were',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'i' },
  'taient'	=> { 'x' => 'were',
 		     'r' => 'tre',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'i' },
  't'		=> { 'x' => 'been',
		     'r' => 'tre',
		     't' => 'v',
		     'c' => 'pp' },
 #  avoir - have
 'avoir'	=> { 'x' => 'have',
		     't' => 'v' },
  'ai'		=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'as'		=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'p' },
  'a'		=> { 'x' => 'has',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'p' },
  'avons'	=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'p' },
  'avez'	=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'p' },
  'ont'		=> { 'x' => 'has',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'p' },
  'avais'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
#		     'p' => '1',
		     'n' => 's',
		     'c' => 'i' },
#  'avais'		=> { 'x' => 'had',
#		     'r' => 'avoir',
#		     't' => 'v',
#		     'p' => '2',
#		     'n' => 's',
#		     'c' => 'i' },
  'avait'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'i' },
  'avions'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'i' },
  'aviez'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'i' },
  'avaient'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'i' },
  'aie'		=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'ps' },
  'aies'	=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'ps' },
  'ait'		=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'ps' },
  'ayons'	=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'ps' },
  'ayez'	=> { 'x' => 'have',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'ps' },
  'aient'	=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'ps' },
  'eu'		=> { 'x' => 'had',
		     'r' => 'avoir',
		     't' => 'v',
		     'c' => 'pp' },
 #  faire - do, make
 'devoir'	=> { 'x' => 'owe',
		     '#' => 'irreg.; have to',
		     't' => 'v' },
  'd'		=> { 'x' => 'owed',
		     '#' => 'had to',
		     'r' => 'devoir' },
 'faire'	=> { 'x' => 'do',
		     '#' => 'make; irreg.',
		     't' => 'v' },
  'fait'	=> { 'x' => 'done',
		     '#' => 'made',
		     'r' => 'faire',
		     't' => 'v',
		     'c' => 'pp' },
 'pouvoir'	=> { 'x' => 'can',
		     '#' => 'irreg; modal; be able to',
		     't' => 'v' },
  'pu'		=> { 'x' => 'been able to',
		     'r' => 'pouvoir',
		     't' => 'v',
		     'c' => 'pp' },
 'vouloir'	=> { 'x' => 'want',
		     '#' => 'wish, be willing, mean, consent, need, try',
 		     't' => 'v' },
  'voulu'	=> { 'x' => 'wanted',
		     'r' => 'vouloir',
		     't' => 'v',
		     'c' => 'pp' },
 #  aller - go
 'aller'	=> { 'x' => 'go',
		     't' => 'v' },

 # Vocabulary
 'absence'	=> { 'x' => 'absence',
		     't' => 'n' },
 'acheter'	=> { 'x' => 'buy',
		     't' => 'v' },
 'accs'	=> { 'x' => 'access',
		     't' => 'n' },
 'actuel'	=> { 'x' => 'current',
		     't' => 'a' },
 'adresse'	=> { 'x' => 'address',
		     't' => 'n' },
 'afrique'	=> { 'x' => 'africa',
		     '#' => 'proper?; place',
 		     't' => 'n' },
 'aider'	=> { 'x' => 'help',
		     't' => 'v' },
 'aimer'	=> { 'x' => 'like',
		     '#' => 'love',
		     't' => 'v' },
 'allemagne'	=> { 'x' => 'germany',
 		     't' => 'n' },
 'allemand'	=> { 'x' => 'german',
		     '#' => 'noun & adj.' },
 'alphabet'	=> { 'x' => 'alphabet',
 		     't' => 'n',
		     'g' => 'm' },
 'ami'		=> { 'x' => 'friend',
 		     't' => 'n',
		     'g' => 'm' },
 'amor'		=> { 'x' => 'love',
		     '#' => 'lover',
 		     't' => 'n' },
 'an'		=> { 'x' => 'year',
 		     't' => 'n',
		     'g' => 'm' },
 'anglais'	=> { 'x' => 'english',
		     '#' => 'adj. & noun' },
 'angleterre'	=> { 'x' => 'england',
		     '#' => 'proper, place',
		     't' => 'n' },
 'arabe'	=> { 'x' => 'arabic',
		     '#' => 'adj. & noun' },
 'argent'	=> { 'x' => 'money',
		     't' => 'n' },
 'armnien'	=> { 'x' => 'armenian',
		     '#' => 'adj. & noun' },
 'article'	=> { 'x' => 'article',
		     't' => 'n' },
 'astronaute'	=> { 'x' => 'astronaut',
 		     't' => 'n' },
 'attention'	=> { 'x' => 'caution',
		     '#' => 'attention?',
		     't' => 'n' },
 'attendre'	=> { 'x' => 'wait for',
		     '#' => 'multiword',
		     't' => 'v' },
 'australie'	=> { 'x' => 'australia',
 		     't' => 'n' },
 'autriche'	=> { 'x' => 'austria',
		     '#' => 'proper, place',
		     't' => 'n' },
 'avion'	=> { 'x' => 'aeroplane',
		     '#' => 'airplane, aircraft, etc',
		     't' => 'n',
		     'g' => 'm' },
 'beau'		=> { 'x' => 'beautiful',
		     '#' => 'irreg.',
 		     't' => 'a' },
  'bel'		=> { 'x' => 'beautiful',
		     'r' => 'beau',
		     't' => 'a',
		     'g' => 'm',
		     'n' => 'p' },
  'belle'	=> { 'x' => 'beautiful',
		     'r' => 'beau',
		     't' => 'a',
		     'g' => 'f',
		     'n' => 'p' },
 'belgique'	=> { 'x' => 'belgium',
		     '#' => 'proper, place',
		     't' => 'n' },
 'bibliothqu'	=> { 'x' => 'library',
 		     't' => 'n',
		     'g' => 'f' },
 'bien'		=> { 'x' => 'well',
		     '#' => 'irreg.; cf bon/bonne',
 		     't' => 'adv' },
  'mieux'	=> { 'x' => 'better',
  		     't' => 'adv' },
 'bijou'	=> { 'x' => 'jewell',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'x' },
 'biologie'	=> { 'x' => 'biology',
		     't' => 'n',
		     'g' => 'f' },
 'blanc'	=> { 'x' => 'white',
		     '#' => 'irreg.',
 		     't' => 'a',
		     'g' => 'm' },
  'blanche'	=> { 'x' => 'white',
  		     'r' => 'blanc',
		     't' => 'a',
		     'g' => 'f' },
 'boeuf'	=> { 'x' => 'bull',
		     '#' => 'beef',
		     't' => 'n',
		     'g' => 'm' },
 'boire'	=> { 'x' => 'drink',
		     't' => 'v' },
  'bu'		=> { 'x' => 'drunk',
		     'r' => 'boire',
		     't' => 'v',
		     'c' => 'pp' },
 'bon'		=> { 'x' => 'good',
		     '#' => 'irreg.',
		     't' => 'a',
		     '#' => 'cf bien',
		     'g' => 'm' },
  'bonne'	=> { 'x' => 'good',
  		     'r' => 'bon',
		     't' => 'a',
		     'g' => 'f' },
  'meilleur'	=> { 'x' => 'better',
		     '#' => 'best',
  		     't' => 'a',
		     '#' => 'also noun' },
 'bord'		=> { 'x' => 'edge',
		     '#' => 'board in "a bord"',
 		     't' => 'n',
		     'g' => 'm' },
 'boucle'	=> { 'x' => 'loop',
		     '#' => 'curl; buckle',
 		     't' => 'n',
		     'g' => 'f' },
 'brsil'	=> { 'x' => 'brazil',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'canada'	=> { 'x' => 'canada',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'm' },
 'cartographie'	=> { 'x' => 'cartography',
 		     't' => 'n' },
 'catalan'	=> { 'x' => 'catalan',
		     '#' => 'adj. & noun' },
 'cendre'	=> { 'x' => 'ash',
 		     't' => 'n',
		     'g' => 'f' },
 'chat'		=> { 'x' => 'cat',
 		     't' => 'n',
		     'g' => 'm' },
 'chemin'	=> { 'x' => 'path',
		     '#' => 'way',
 		     't' => 'n',
		     'g' => 'm' },
 'chercher'	=> { 'x' => 'search',
		     't' => 'v' },
 'cheveu'	=> { 'x' => 'hair',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'x' },
 'chevelure'	=> { 'x' => 'hair',
 		     't' => 'n',
		     'g' => 'f' },
 'chien'	=> { 'x' => 'dog',
 		     't' => 'n',
		     'g' => 'm' },
 'chienne'	=> { 'x' => 'bitch',
 		     't' => 'n',
		     'g' => 'f' },
 'chinois'	=> { 'x' => 'chinese',
		     '#' => 'adj. & noun' },
 'chocolat'	=> { 'x' => 'chocolate',
		     't' => 'n' },
 'choisir'	=> { 'x' => 'choose',
		     't' => 'v' },
 'chose'	=> { 'x' => 'thing',
		     '#' => 'sometimes masc.',
		     't' => 'n',
		     'g' => 'f' },
 'cit'		=> { 'x' => 'city',
		     't' => 'n',
		     'g' => 'f' },
 'cl'		=> { 'x' => 'key',
		     't' => 'n' },
 'coin'		=> { 'x' => 'corner',
		     '#' => 'spot',
		     't' => 'n',
		     'g' => 'm' },
 'compatibilit'	=> { 'x' => 'compatibility',
			     't' => 'n' },
 'complet'	=> { 'x' => 'complete',
		     't' => 'a' },
 'compromis'	=> { 'x' => 'compromise',
		     't' => 'n' },
 'connatre'	=> { 'x' => 'know',
		     '#' => 'conocer',
		     't' => 'v' },
 'coq'		=> { 'x' => 'cock',
		     '#' => 'rooster',
		     't' => 'n',
		     'g' => 'm' },
 'coren'	=> { 'x' => 'korean',
		     '#' => 'adj. & noun' },
 'cosmonaute'	=> { 'x' => 'cosmonaut',
 		     't' => 'n' },
 'couleur'	=> { 'x' => 'colour',
		     '#' => 'suit (cards)',
 		     't' => 'n',
		     'g' => 'f' },
 'coup'		=> { 'x' => 'blow',
 		     't' => 'n',
		     'g' => 'm' },
 'courir'	=> { 'x' => 'run',
		     '#' => 'irreg.',
		     't' => 'v' },
  'couru'	=> { 'x' => 'run',
		     'r' => 'courir',
		     't' => 'v',
		     'c' => 'pp' },
 'cousin'	=> { 'x' => 'cousin',
		     't' => 'n',
		     'g' => 'm' },
 'cousine'	=> { 'x' => 'cousin',
		     't' => 'n',
		     'g' => 'f' },
 'couvent'	=> { 'x' => 'convent',
 		     't' => 'n',
		     'g' => 'm' },
 'craindre'	=> { 'x' => 'fear',
		     '#' => 'irreg.',
		     't' => 'v' },
  'craint'	=> { 'x' => 'feared',
		     'r' => 'craindre',
		     't' => 'v',
		     'c' => 'pp' },
 'crne'	=> { 'x' => 'skull',
 		     't' => 'n',
		     'g' => 'm' },
 'cration'	=> { 'x' => 'creation',
		     't' => 'n' },
 'crer'	=> { 'x' => 'create',
		     't' => 'v' },
 'croire'	=> { 'x' => 'believe',
		     '#' => 'irreg.',
		     't' => 'v' },
  'cru'		=> { 'x' => 'believed',
		     'r' => 'croire',
		     't' => 'v',
		     'c' => 'pp' },
 'croissant'	=> { 'x' => 'croissant',
		     '#' => 'crescent',
 		     't' => 'n',
		     'g' => 'm' },
 'dactilo'	=> { 'x' => 'typist',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'f' },
 'dalle'	=> { 'x' => 'slab',
 		     't' => 'n',
		     'g' => 'f' },
 'danois'	=> { 'x' => 'danish',
		     '#' => 'adj. & nm' },
 'danser'	=> { 'x' => 'dance',
		     't' => 'v' },
 'dcouvrir'	=> { 'x' => 'discover',
		     't' => 'v' },
 'ddicace'	=> { 'x' => 'dedication',
 		     't' => 'n',
		     'g' => 'f' },
 'ddier'	=> { 'x' => 'dedicate',
		     '#' => 'consecrate',
 		     't' => 'v' },
 'dent'		=> { 'x' => 'tooth',
 		     't' => 'n',
		     'g' => 'f' },
 'descendre'	=> { 'x' => 'descend',
		     't' => 'v' },
 'dictionnaire'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 'dner'	=> { 'x' => 'dine',
		     't' => 'v' },
 'dire'		=> { 'x' => 'say',
		     '#' => 'irreg.',
		     't' => 'v' },
  'dit'		=> { 'x' => 'said',
		     'r' => 'dire',
		     't' => 'v',
		     'c' => 'pp' },
 'disque'	=> { 'x' => 'disk',
		     '#' => 'disc',
		     't' => 'n' },
 'dossier'	=> { 'x' => 'file',
		     '#' => 'computer',
		     't' => 'n' },
 'dur'		=> { 'x' => 'hard',
		     't' => 'a' },
 'eau'		=> { 'x' => 'water',
		     '#' => 'pl. has other meaning',
		     't' => 'n',
		     'g' => 'f' },
 'cole'	=> { 'x' => 'school',
 		     't' => 'n',
		     'g' => 'f' },
 'crire'	=> { 'x' => 'write',
		     '#' => 'irreg.',
		     't' => 'v' },
  'crit'	=> { 'x' => 'written',
		     'r' => 'crire',
		     't' => 'v',
		     'c' => 'pp' },
 'egypte'	=> { 'x' => 'egypt',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'lectricit'	=> { 'x' => 'electricity',
 		     't' => 'n' },
 'enfant'	=> { 'x' => 'child',
		     '#' => 'masc & fem!',
 		     't' => 'n' },
 'ensevelir'	=> { 'x' => 'bury',
 		     't' => 'v' },
 'entendre'	=> { 'x' => 'listen',
		     't' => 'v' },
 'esclave'	=> { 'x' => 'slave',
		     '#' => 'gender = m & f',
 		     't' => 'n' },
 'espagne'	=> { 'x' => 'spain',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'espagnol'	=> { 'x' => 'spanish',
		     '#' => 'noun & adj.' },
 'espranto'	=> { 'x' => 'esperanto',
 		     't' => 'n' },
 'espoir'	=> { 'x' => 'hope',
 		     't' => 'n',
		     'g' => 'm' },
 'tat'		=> { 'x' => 'state',
		     '#' => 'condition',
 		     't' => 'n',
		     'g' => 'm' },
 'toile'	=> { 'x' => 'star',
 		     't' => 'n',
		     'g' => 'f' },
 'exemple'	=> { 'x' => 'example',
		     't' => 'n' },
 'evidemment'	=> { 'x' => 'obviously',
		     '#' => 'evidently',
		     't' => 'adv' },
 'facile'	=> { 'x' => 'easy',
		     't' => 'a' },
 'femme'	=> { 'x' => 'woman',
		     't' => 'n',
		     'g' => 'f' },
 'fiable'	=> { 'x' => 'reliable',
 		     't' => 'a' },
 'fils'		=> { 'x' => 'boy',
		     't' => 'son',
		     'g' => 'm' },
 'fille'	=> { 'x' => 'girl',
		     '#' => 'daugther',
		     't' => 'n',
		     'g' => 'f' },
 'finir'	=> { 'x' => 'finish',
		     't' => 'v' },
 'finlandais'	=> { 'x' => 'finnish',
		     '#' => 'noun & adj.' },
 'fonctionnel'	=> { 'x' => 'functional',
		     't' => 'a' },
 'fraise'	=> { 'x' => 'strawberry',
 		     't' => 'n',
		     'g' => 'f' },
 'franais'	=> { 'x' => 'french',
		     '#' => 'noun & adj.' },
 'france'	=> { 'x' => 'france',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'f' },
 'frre'	=> { 'x' => 'brother',
		     't' => 'n',
		     'g' => 'm' },
 'fromage'	=> { 'x' => 'cheese',
 		     't' => 'n',
		     'g' => 'm' },
 'galaxie'	=> { 'x' => 'galaxy',
 		     't' => 'n' },
 'galicien'	=> { 'x' => 'galician',
		     '#' => 'noun & adj.' },
 'garon'	=> { 'x' => 'boy',
		     '#' => 'waiter',
 		     't' => 'n',
		     'g' => 'm' },
 'gorgien'	=> { 'x' => 'georgian',
		     '#' => 'noun & adj.' },
 'grand'	=> { 'x' => 'big',
		     '#' => 'great',
		     't' => 'a' },
 'gratuit'	=> { 'x' => 'free',
		     '#' => 'as in beer',
		     't' => 'a' },
 'grec'		=> { 'x' => 'greek',
		     '#' => 'noun & adj.' },
 'guenon'	=> { 'x' => 'ape',
		     '#' => 'female; cf. singe',
		     't' => 'n',
		     'g' => 'f' },
 'guillemet'	=> { 'x' => 'inverted comma',
		     '#' => 'multiword',
 		     't' => 'n',
		     'g' => 'm' },
 'hbreu'	=> { 'x' => 'hebrew',
		     '#' => 'noun & adj.' },
 'hrone'	=> { 'x' => 'heroine',
		     't' => 'n',
		     'g' => 'f' },
 'hros'	=> { 'x' => 'hero',
		     't' => 'n',
		     'g' => 'm' },
 'heure'	=> { 'x' => 'hour',
 		     't' => 'n',
		     'g' => 'f' },
 'hindi'	=> { 'x' => 'hindi',
		     '#' => 'adj. & noun' },
 'hollandais'	=> { 'x' => 'dutch',
		     '#' => 'adj. & noun' },
 'homme'	=> { 'x' => 'man',
		     't' => 'n',
		     'g' => 'm' },
 'hongrois'	=> { 'x' => 'hungarian',
		     '#' => 'adj. & noun' },
 'hpital'	=> { 'x' => 'hospital',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'aux' },
 'hte'		=> { 'x' => 'host',
		     't' => 'n',
		     'g' => 'm' },
 'htel'	=> { 'x' => 'hotel',
		     't' => 'n',
		     'g' => 'm' },
 'ide'		=> { 'x' => 'idea',
 		     't' => 'n',
		     'g' => 'f' },
 'illimit'	=> { 'x' => 'unlimited',
		     't' => 'a' },
 'intgr'	=> { 'x' => 'integrated',
		     't' => 'a' },
 'irlande'	=> { 'x' => 'ireland',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'islandais'	=> { 'x' => 'icelandic',
		     '#' => 'adj. & noun?' },
 'Italie'	=> { 'x' => 'italy',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'Italien'	=> { 'x' => 'italian',
		     '#' => 'noun & adj.' },
 'japon'	=> { 'x' => 'japan',
 		     't' => 'n' },
 'japonais'	=> { 'x' => 'japanese',
		     '#' => 'adj. & noun' },
 'joie'		=> { 'x' => 'joy',
		     't' => 'n',
		     'g' => 'f' },
 'jouer'	=> { 'x' => 'play',
 		     't' => 'v' },
 'jour'		=> { 'x' => 'day',
		     '#' => 'daylight, light, hole, gap',
 		     't' => 'n',
		     'g' => 'm' },
 'lait'		=> { 'x' => 'milk',
		     't' => 'n',
		     'g' => 'm' },
 'latin'	=> { 'x' => 'latin',
		     '#' => 'noun & adj.' },
 'lopard'	=> { 'x' => 'leopard',
 		     't' => 'n',
		     'g' => 'm' },
 'letton'	=> { 'x' => 'latvian',
		     '#' => 'noun & adj.?' },
 'lettre'	=> { 'x' => 'letter',
		     '#' => 'alphabet & mail?',
 		     't' => 'n',
		     'g' => 'f' },
 'librairie'	=> { 'x' => 'bookshop',
 		     't' => 'n',
		     'g' => 'f' },
 'liechtenstein'	=> { 'x' => 'liechtenstein',
		     '#' => 'proper, place',
			     't' => 'n' },
 'lire'		=> { 'x' => 'read',
		     '#' => 'irreg.',
		     't' => 'v' },
 'lituanien'	=> { 'x' => 'lithuanian',
		     '#' => 'noun & adj.' },
  'lu'		=> { 'x' => 'read',
		     'r' => 'lire',
		     't' => 'v',
		     'c' => 'pp' },
 'livre'	=> { 'x' => 'book',
		     '#' => 'fem. = pound',
		     't' => 'n',
		     'g' => 'm' },
 'logiciel'	=> { 'x' => 'software',
		     't' => 'n' },
 'lune'		=> { 'x' => 'moon',
		     '#' => 'no pl.',
		     't' => 'n',
		     'g' => 'f' },
 'luxembourg'	=> { 'x' => 'luxembourg',
		     '#' => 'proper, place',
		     't' => 'n' },
 'magie'	=> { 'x' => 'magic',
 		     't' => 'n',
		     'g' => 'f' },
 'magique'	=> { 'x' => 'magic',
 		     't' => 'a' },
 'maison'	=> { 'x' => 'house',
		     '#' => 'home',
 		     't' => 'n',
		     'g' => 'f' },
 'matre'	=> { 'x' => 'master',
		     '#' => 'cf matresse',
 		     't' => 'n',
		     'g' => 'm' },
 'matresse'	=> { 'x' => 'mistress',
		     '#' => 'cf matre',
 		     't' => 'n',
		     'g' => 'f' },
 'manger'	=> { 'x' => 'eat',
		     't' => 'v' },
 'march'	=> { 'x' => 'market',
		     't' => 'n' },
 'mmoire'	=> { 'x' => 'memory',
		     't' => 'n' },
 'mre'		=> { 'x' => 'mother',
		     't' => 'n',
		     'g' => 'f' },
 'merde'	=> { 'x' => 'shit',
 		     't' => 'n' },
 'mettre'	=> { 'x' => 'put',
		     '#' => 'irreg.',
		     't' => 'v' },
 'mexique'	=> { 'x' => 'mexico',
		     '#' => 'proper, place',
 		     't' => 'n' },
  'mis'		=> { 'x' => 'put',
		     'r' => 'mettre',
		     't' => 'v',
		     'c' => 'pp' },
 'minute'	=> { 'x' => 'minute',
 		     't' => 'n' },
 'mois'		=> { 'x' => 'month',
 		     't' => 'n',
		     'g' => 'm' },
 'mot'		=> { 'x' => 'word',
 		     't' => 'n',
		     'g' => 'm' },
 'mourir'	=> { 'x' => 'die',
		     '#' => 'irreg.',
 		     't' => 'v' },
  'mort'	=> { 'x' => 'dead',
		     'r' => 'mourir',
		     't' => 'v',
		     'c' => 'pp' },
 'moyen'	=> { 'x' => 'average',
		     '#' => 'medium; irreg.?',
 		     't' => 'a',
		     'g' => 'm' },
  'moyenne'	=> { 'x' => 'average',
  		     'r' => 'moyen',
		     't' => 'a',
		     'g' => 'f' },
 'natre'	=> { 'x' => 'be born',
		     '#' => '(2 words); transl. to passive in English; irreg.',
		     't' => 'v' },
  'n'		=> { 'x' => 'born',
		     'r' => 'natre',
		     't' => 'v',
		     'c' => 'pp' },
 'nectarine'	=> { 'x' => 'nectarine',
 		     't' => 'n',
		     'g' => 'f' },
 'neveu'	=> { 'x' => 'nephew',
		     't' => 'n',
		     'g' => 'm' },
 'nez'		=> { 'x' => 'nose',
 		     't' => 'n',
		     'g' => 'm' },
 'nice'	=> { 'x' => 'niece',
		     't' => 'n',
		     'g' => 'f' },
 'niveau'	=> { 'x' => 'level',
		     't' => 'n' },
 'nom'		=> { 'x' => 'name',
		     '#' => '"noun"',
 		     't' => 'n',
		     'g' => 'm' },
 'norvge'	=> { 'x' => 'norway',
		     '#' => 'proper, place',
		     't' => 'n' },
 'norvgien'	=> { 'x' => 'norwegian',
		     '#' => 'noun & adj' },
 'nouveau'	=> { 'x' => 'new',
		     't' => 'a' },
 'nuit'		=> { 'x' => 'night',
 		     't' => 'n',
		     'g' => 'f' },
 'oeil'		=> { 'x' => 'eye',
		     '#' => 'irreg.',
		     't' => 'n',
		     'g' => 'm' },
  'yeux'	=> { 'x' => 'eyes',
		     '#' => 'oeils also exists',
		     'r' => 'oeil',
		     't' => 'n',
		     'g' => 'm',
		     'n' => 'p' },
 'oeuf'		=> { 'x' => 'egg',
 		     't' => 'n',
		     'g' => 'm' },
 'oeuvre'	=> { 'x' => 'work',
		     '#' => 'masc. in some expressions',
		     't' => 'n',
		     'g' => 'f' },
 'oncle'	=> { 'x' => 'uncle',
		     't' => 'n',
		     'g' => 'm' },
 'ongle'	=> { 'x' => 'nail',
 		     't' => 'n',
		     'g' => 'm' },
 'orange'	=> { 'x' => 'orange',
		     '#' => 'invar.',
		     't' => 'a' },
 'oreille'	=> { 'x' => 'ear',
		     't' => 'n' },
 'ouvrir'	=> { 'x' => 'open',
		     '#' => 'irreg.',
		     't' => 'v' },
  'ouvert'	=> { 'x' => 'opened',
		     'r' => 'ouvrir',
		     't' => 'v',
		     'c' => 'pp' },
 'paix'		=> { 'x' => 'peace',
		     't' => 'n',
		     'g' => 'f' },
 'papier'	=> { 'x' => 'paper',
 		     't' => 'n' },
 'parfait'	=> { 'x' => 'perfect',
		     't' => 'a' },
 'parler'	=> { 'x' => 'speak',
		     '#' => 'talk',
		     't' => 'v' },
 'pays'		=> { 'x' => 'country',
		     't' => 'n',
		     'g' => 'm' },
 'pre'		=> { 'x' => 'father',
		     't' => 'n',
		     'g' => 'm' },
 'perdre'	=> { 'x' => 'lose',
		     't' => 'v' },
 'persan'	=> { 'x' => 'persian',
		     '#' => 'adj & noun; farsi' },
 'personne'	=> { 'x' => 'person',
 		     't' => 'n',
		     'g' => 'f' },
 'petit'	=> { 'x' => 'little',
		     '#' => 'small',
 		     't' => 'a' },
 'phrase'	=> { 'x' => 'sentence',
		     '#' => 'gramm.',
 		     't' => 'n',
		     'g' => 'f' },
 'pied'		=> { 'x' => 'foot',
 		     't' => 'n',
		     'g' => 'm' },
 'pioche'	=> { 'x' => 'pick',
 		     't' => 'n',
		     'g' => 'f' },
 'plan'		=> { 'x' => 'map',
		     't' => 'n' },
 'plupart'	=> { 'x' => 'majority',
		     't' => 'n' },
 'polonais'	=> { 'x' => 'polish',
		     '#' => 'noun & adj.' },
 'portugais'	=> { 'x' => 'portuguese',
		     '#' => 'noun & adj.' },
 'portugal'	=> { 'x' => 'portugal',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'potiron'	=> { 'x' => 'pumpkin',
 		     't' => 'n',
		     'g' => 'm' },
 'pouce'	=> { 'x' => 'thumb',
		     '#' => 'inch?',
 		     't' => 'n',
		     'g' => 'm' },
 'poule'	=> { 'x' => 'hen',
		     '#' => 'chicken as in meat',
		     't' => 'n',
		     'g' => 'f' },
 'prendre'	=> { 'x' => 'take',
		     '#' => 'irreg.',
		     't' => 'v' },
  'pris'	=> { 'x' => 'taken',
		     'r' => 'prendre',
		     't' => 'v',
		     'c' => 'pp' },
 'prince'	=> { 'x' => 'prince',
 		     't' => 'n',
		     'g' => 'm' },
 'prochain'	=> { 'x' => 'next',
 		     't' => 'a' },
 'rapide'	=> { 'x' => 'rapid',
		     '#' => 'quick',
		     't' => 'a' },
 'recevoir'	=> { 'x' => 'receive',
		     '#' => 'irreg?',
 		     't' => 'v' },
 'reu'		=> { 'x' => 'receipt',
		     '#' => 'also pp. of recevoir',
 		     't' => 'n',
		     'g' => 'm' },
 'reine'	=> { 'x' => 'queen',
		     't' => 'n',
		     'g' => 'f' },
 'rendre'	=> { 'x' => 'return',
		     't' => 'v' },
 'rpondre'	=> { 'x' => 'answer',
		     't' => 'v' },
 'russir'	=> { 'x' => 'succeed',
		     't' => 'v' },
 'revoir'	=> { 'x' => 'see again',
		     '#' => 'irreg.; revise',
 		     't' => 'v' },
 'richesse'	=> { 'x' => 'richness',
		     't' => 'n' },
 'rigide'	=> { 'x' => 'rigid',
		     '#' => 'stiff',
 		     't' => 'a' },
 'roi'		=> { 'x' => 'king',
		     't' => 'n',
		     'g' => 'm' },
 'russe'	=> { 'x' => 'russian',
		     '#' => 'noun & adj.' },
 'russie'	=> { 'x' => 'russia',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'savoir'	=> { 'x' => 'know',
		     '#' => 'irreg.',
		     't' => 'v',
		     'x-de' => 'kennen',
		     'x-es' => 'saber' },
  'su'		=> { 'x' => 'known',
		     'r' => 'savoir',
		     't' => 'v',
		     'c' => 'pp' },
 'slection'	=> { 'x' => 'selection',
		     't' => 'n' },
 'semaine'	=> { 'x' => 'week',
		     't' => 'n' },
 'srieux'	=> { 'x' => 'serious',
		     '#' => 'f. srieuse',
 		     't' => 'a' },
 'serpent'	=> { 'x' => 'snake',
		     '#' => 'serpent',
		     't' => 'n',
		     'g' => 'm' },
 'singapour'	=> { 'x' => 'singapore',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'singe'	=> { 'x' => 'ape',
		     '#' => 'male',
		     't' => 'n',
		     'g' => 'm' },
 'slovaque'	=> { 'x' => 'slovakian',
		     '#' => 'noun & adj.' },
 'slovne'	=> { 'x' => 'slovenian',
		     '#' => 'noun & adj.' },
 'soeur'	=> { 'x' => 'sister',
		     't' => 'n',
		     'g' => 'f' },
 'soleil'	=> { 'x' => 'sun',
		     '#' => 'no pl.',
		     't' => 'n',
		     'g' => 'm' },
 'solidarit'	=> { 'x' => 'solidarity',
 		     't' => 'n' },
 'souris'	=> { 'x' => 'mouse',
		     't' => 'n',
		     'g' => 'f' },
 'sude'	=> { 'x' => 'sweden',
		     '#' => 'proper, place',
		     't' => 'n',
		     'g' => 'f' },
 'sudois'	=> { 'x' => 'swedish',
		     '#' => 'noun & adj.' },
 'suisse'	=> { 'x' => 'switzerland',
		     '#' => 'proper, place',
		     't' => 'n' },
 'suivre'	=> { 'x' => 'follow',
		     '#' => 'irreg.',
		     't' => 'v' },
  'suivi'	=> { 'x' => 'followed',
		     'r' => 'suivre',
		     't' => 'v',
		     'c' => 'pp' },
 'taiwan'	=> { 'x' => 'taiwan',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'tante'	=> { 'x' => 'aunt',
		     't' => 'n',
		     'g' => 'f' },
 'tard'		=> { 'x' => 'late',
		     '#' => 'not adj.',
 		     't' => 'adv' },
 'tchque'	=> { 'x' => 'czech',
		     '#' => 'adj. and noun?' },
 'tlcharger'	=> { 'x' => 'download',
		     't' => 'v' },
 'tenir'	=> { 'x' => 'hold',
		     '#' => 'irreg.',
		     't' => 'v' },
  'tenu'	=> { 'x' => 'held',
		     'r' => 'tenir',
		     't' => 'v',
		     'c' => 'pp' },
 'tentation'	=> { 'x' => 'temptation',
 		     't' => 'n' },
 'texte'	=> { 'x' => 'text',
		     't' => 'n' },
 'tha'		=> { 'x' => 'thai',
		     '#' => 'noun & adj.' },
 'thailande'	=> { 'x' => 'thailand',
		     '#' => 'proper; place',
		     't' => 'n' },
 'toilette'	=> { 'x' => 'toilet',
 		     't' => 'n',
		     'g' => 'f' },
 'traduction'	=> { 'x' => 'translation',
		     't' => 'n' },
 'turc'		=> { 'x' => 'turkish',
		     '#' => 'noun & adj.' },
 'uruguay'	=> { 'x' => 'uruguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'vache'	=> { 'x' => 'cow',
		     't' => 'n',
		     'g' => 'f' },
 'vhicule'	=> { 'x' => 'vehicle',
 		     't' => 'n',
		     'g' => 'm' },
 'vlo'		=> { 'x' => 'bicycle',
		     '#' => 'bike',
 		     't' => 'n',
		     'g' => 'm' },
 'vendre'	=> { 'x' => 'sell',
		     't' => 'v' },
 'venir'	=> { 'x' => 'come',
		     '#' => 'irreg.',
		     't' => 'v' },
  'venu'	=> { 'x' => 'come',
		     'r' => 'venir',
		     't' => 'v',
		     'c' => 'pp' },
 'verbe'	=> { 'x' => 'verb',
 		     't' => 'n',
		     'g' => 'm' },
 'verre'	=> { 'x' => 'glass',
		     '#' => 'both senses as in english',
 		     't' => 'n',
		     'g' => 'm' },
 'version'	=> { 'x' => 'version',
 		     't' => 'n' },
 'vie'		=> { 'x' => 'life',
 		     't' => 'n' },
 'vietnam'	=> { 'x' => 'vietnam',
		     '#' => 'proper, place',
		     't' => 'n' },
 'vietnamien'	=> { 'x' => 'vietnamese',
		     '#' => 'noun & adj.' },
 'voir'		=> { 'x' => 'see',
		     '#' => 'irreg.',
		     't' => 'v' },
 'voiture'	=> { 'x' => 'car',
 		     't' => 'n',
		     'g' => 'f' },
 'voix'		=> { 'x' => 'voice',
		     't' => 'n',
		     'g' => 'f' },
 'voyage'	=> { 'x' => 'voyage',
		     't' => 'n',
		     'g' => 'm' },
  'vu'		=> { 'x' => 'seen',
		     'r' => 'voir',
		     't' => 'v',
		     'c' => 'pp' },
);
}

1;

