# vi: ts=8 sw=8

package Tongues::Italiano;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun gender
 # Noun plural
 [ 'i',		'o',	'',		's',	'n' ],	# masculine
 [ 'e',		'a',	'',		's',	'n' ],	# feminine
 [ 'i',		'e',	'',		's',	'n' ],	# either
 # Adjectives
 [ 'i',		'o',	'',		'',	'a' ],	# masculine plural
 [ 'a',		'o',	'',		'',	'a' ],	# feminine singular
 [ 'e',		'o',	'',		'',	'a' ],	# feminine plural
 [ 'i',		'e',	'',		'',	'a' ],	# -e adjectives plural
 # Verb past participle
 [ 'ato',	'are',	'',		'ed',	'v' ],
 [ 'uto',	'ere',	'',		'ed',	'v' ],
 [ 'ito',	'ire',	'',		'ed',	'v' ],
 # Verb gerund
 # Verb present participle
 [ 'ando',	'are',	'',		'ing',	'v' ],
 [ 'endo',	'ere',	'',		'ing',	'v' ],
 [ 'endo',	'ire',	'',		'ing',	'v' ],
 # Verb present
 [ 'o',		'are',	'(i) ',		'',	'v' ],
 [ 'o',		'ere',	'(i) ',		'',	'v' ],
 [ 'o',		'ire',	'(i) ',		'',	'v' ],
 [ 'isco',	'ire',	'(i) ',		'',	'v' ],
 [ 'i',		'are',	'(you) ',	'',	'v' ],
 [ 'i',		'ere',	'(you) ',	'',	'v' ],
 [ 'i',		'ire',	'(you) ',	'',	'v' ],
 [ 'isci',	'ire',	'(you) ',	'',	'v' ],
 [ 'a',		'are',	'(he) ',	's',	'v' ],
 [ 'e',		'ere',	'(he) ',	's',	'v' ],
 [ 'e',		'ire',	'(he) ',	's',	'v' ],
 [ 'isce',	'ire',	'(he) ',	's',	'v' ],
 [ 'iamo',	'are',	'(we) ',	'',	'v' ],
 [ 'iamo',	'ere',	'(we) ',	'',	'v' ],
 [ 'iamo',	'ire',	'(we) ',	'',	'v' ],
 [ 'ate',	'are',	'(you) ',	'',	'v' ],
 [ 'ete',	'ere',	'(you) ',	'',	'v' ],
 [ 'ite',	'ire',	'(you) ',	'',	'v' ],
 [ 'ano',	'are',	'(they) ',	'',	'v' ],
 [ 'ono',	'ere',	'(they) ',	'',	'v' ],
 [ 'ono',	'ire',	'(they) ',	'',	'v' ],
 [ 'iscono',	'ire',	'(they) ',	'',	'v' ],
 # Verb present subjunctive, imperitive
 # Verb preterite
 [ 'ai',	'are',	'(I) ',		'ed',	'v' ],
 [ 'ei',	'ere',	'(I) ',		'ed',	'v' ],
 [ 'ii',	'ire',	'(I) ',		'ed',	'v' ],
 [ 'asti',	'are',	'(you) ',	'ed',	'v' ],
 [ 'esti',	'ere',	'(you) ',	'ed',	'v' ],
 [ 'isti',	'ire',	'(you) ',	'ed',	'v' ],
 [ '',		'are',	'(he) ',	'ed',	'v' ],
 [ '',		'ere',	'(he) ',	'ed',	'v' ],
 [ '',		'ire',	'(he) ',	'ed',	'v' ],
 [ 'ammo',	'are',	'(we) ',	'ed',	'v' ],
 [ 'emmo',	'ere',	'(we) ',	'ed',	'v' ],
 [ 'immo',	'ire',	'(we) ',	'ed',	'v' ],
 [ 'aste',	'are',	'(you) ',	'ed',	'v' ],
 [ 'este',	'ere',	'(you) ',	'ed',	'v' ],
 [ 'iste',	'ire',	'(you) ',	'ed',	'v' ],
 [ 'arono',	'are',	'(they) ',	'ed',	'v' ],
 [ 'erono',	'ere',	'(they) ',	'ed',	'v' ],
 [ 'irono',	'ire',	'(they) ',	'ed',	'v' ],
 # Verb imperfect
 [ 'avo',	'are',	'(I) ',		'ed',	'v' ],
 [ 'evo',	'ere',	'(I) ',		'ed',	'v' ],
 [ 'ivo',	'ire',	'(I) ',		'ed',	'v' ],
 [ 'avi',	'are',	'(you) ',	'ed',	'v' ],
 [ 'evi',	'ere',	'(you) ',	'ed',	'v' ],
 [ 'ivi',	'ire',	'(you) ',	'ed',	'v' ],
 [ 'avo',	'are',	'(he) ',	'ed',	'v' ],
 [ 'evo',	'ere',	'(he) ',	'ed',	'v' ],
 [ 'ivo',	'ire',	'(he) ',	'ed',	'v' ],
 [ 'iamo',	'are',	'(we) ',	'ed',	'v' ],
 [ 'iamo',	'ere',	'(we) ',	'ed',	'v' ],
 [ 'iamo',	'ire',	'(we) ',	'ed',	'v' ],
 [ 'ate',	'are',	'(you) ',	'ed',	'v' ],
 [ 'ete',	'ere',	'(you) ',	'ed',	'v' ],
 [ 'ite',	'ire',	'(you) ',	'ed',	'v' ],
 [ 'ano',	'are',	'(they) ',	'ed',	'v' ],
 [ 'ono',	'ere',	'(they) ',	'ed',	'v' ],
 [ 'ono',	'ire',	'(they) ',	'ed',	'v' ],
 # Verb future
 # Verb conditional
 # Adverb
 # Higher level transformations
 # Verb to noun
 [ 'azione',	'are',	'',		'ing',	'v' ],
];

#Fields:
# x -> translatioin		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine		default for words ending in a
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative

# Italian to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 'il'		=> { 'x' => 'the',
		     '#' => 'before consonant',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 's' },
 'lo'		=> { 'x' => 'the',
		     '#' => 'before s impure and z',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 's' },
 'l\''		=> { 'x' => 'the',
		     '#' => 'before vowel',
 		     't' => 'art' },
 #     Plural
 'i'		=> { 'x' => 'the',
		     '#' => 'before consonant',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 'p' },
 'gli'		=> { 'x' => 'the',
		     '#' => 'before s impure and z',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 'p' },
 'gl\''		=> { 'x' => 'the',
		     '#' => 'before i',
 		     't' => 'art',
		     'g' => 'm',
		     'n' => 'p' },
 #    Feminine
 #     Singular
 'la'		=> { 'x' => 'the',
		     '#' => 'before consonant',
 		     't' => 'art',
		     'g' => 'f',
		     'n' => 's' },
 #'l\''		=> { 'x' => 'the',
 #		     '#' => 'before vowel' },
 #     Plural
 'le'		=> { 'x' => 'the',
		     '#' => 'before consonant',
 		     't' => 'art',
		     'g' => 'f',
		     'n' => 'p' },
 #'l\''		=> { 'x' => 'the',
 #		     '#' => 'before e' },
 #   Indefinite articles
 'un'		=> { 'x' => 'a',
		     '#' => 'an, one?',
 		     't' => 'art',
		     'g' => 'm' },
 'uno'		=> { 'x' => 'a',
		     '#' => 'an, one?',
 		     't' => 'art',
		     '#' => 'used only before words beginning with "s impure" and z',
		     'g' => 'm' },
 'una'		=> { 'x' => 'a',
		     '#' => 'an, one?',
 		     't' => 'art',
		     '#' => 'used before any consonant',
		     'g' => 'f' },
 'un\''		=> { 'x' => 'a',
		     '#' => 'an, one?',
 		     't' => 'art',
		     '#' => 'used before a vowel',
		     'g' => 'f' },
 #  Pronouns & possessive adjectives
 #   1st person
 'io'		=> { 'x' => 'i',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'noi'		=> { 'x' => 'we',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p' },
 'mio'		=> { 'x' => 'my',
		     '#' => 'mine',
 		     'p' => '1',
		     '#' => 'poss. adj. and poss. pron.',
		     'n' => 's' },
 #   2nd person
 'tu'		=> { 'x' => 'you',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 's' },
 'voi'		=> { 'x' => 'you',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p' },
 'vostro'	=> { 'x' => 'your',
		     '#' => 'yours',
 		     'p' => '2',
		     '#' => 'poss. adj. and poss. pron.',
		     'n' => 's' },
 #   3rd person
 'lui'		=> { 'x' => 'he',
		     '#' => 'informal',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
 'lei'		=> { 'x' => 'she',
		     '#' => 'informal',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'f' },
 'egli'		=> { 'x' => 'he',
		     '#' => 'literary',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
 'ella'		=> { 'x' => 'she',
		     '#' => 'literary',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'f' },
 'esso'		=> { 'x' => 'he/it',
		     '#' => 'things & animals',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
 'essa'		=> { 'x' => 'she/it',
		     '#' => 'things & animals',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'f' },
 'loro'		=> { 'x' => 'they',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p' },
 'essi'		=> { 'x' => 'they',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'g' => 'm' },
 'esse'		=> { 'x' => 'they',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'g' => 'f' },
 'suo'		=> { 'x' => 'his',
 		     'p' => '3',
		     '#' => 'poss. adj. and poss. pron.',
		     'n' => 's' },
 #   Reflexive pronouns
 #'se'		=> { 'x' => 'self',
 #		     '#' => 'also "if"',
 #		     't' => 'pro' },
 's'		=> { 'x' => 'self',
 		     't' => 'pro' },
 'si'		=> { 'x' => 'self',
 		     't' => 'pro' },
 #  Other functional words
 'c\''		=> { 'x' => 'there is',
		     '#' => 'hay' },
 #'ci sono'	=> { 'x' => 'there are' },
 'a' 		=> { 'x' => 'to',
		     '#' => 'at; related to "ad"',
 		     't' => 'p' },
 'ad' 		=> { 'x' => 'to',
		     '#' => 'at; related to "a"',
 		     't' => 'p' },
 'alcuno'	=> { 'x' => 'any',
		     '#' => 'some',
 		     't' => 'a' },
 'che'		=> { 'x' => 'that',
		     '#' => 'who, which' },
 'come'		=> { 'x' => 'how',
		     '#' => 'like, as' },
 'con'		=> { 'x' => 'with',
 		     't' => 'p' },
 'cos'		=> { 'x' => 'thus',
		     '#' => 'cf. es. as',
 		     't' => 'adv' },
 'da'		=> { 'x' => 'by',
		     '#' => 'from',
 		     't' => 'p' },
 'di'		=> { 'x' => 'of',
 		     't' => 'p' },
 'domani'	=> { 'x' => 'tomorrow',
		     '#' => 'adverb or noun?' },
 'e'		=> { 'x' => 'and',
		     '#' => 'also; related to "ed"',
 		     't' => 'conj' },
 'ed'		=> { 'x' => 'and',
		     '#' => 'also; related to "e"',
 		     't' => 'conj' },
 'in'		=> { 'x' => 'in',
		     '#' => 'into',
 		     't' => 'p' },
 'infine'	=> { 'x' => 'at last',
		     '#' => 'multiword',
 		     't' => 'adv' },
 'ma'		=> { 'x' => 'but',
 		     't' => 'conj' },
 'mai'		=> { 'x' => 'never',
		     '#' => 'non...mai' },
 'mentre'	=> { 'x' => 'while',
 		     't' => 'p' },
 'molto'	=> { 'x' => 'very',
		     '#' => 'agreement?' },
 'n'		=> { 'x' => 'nor' },
 'nessuno'	=> { 'x' => 'no',
		     '#' => 'related to uno/un/una/un\'; adj. and pron.="nobody"' },
  'nessun'	=> { 'x' => 'no' },
  'nessuna'	=> { 'x' => 'no' },
  'nessun\''	=> { 'x' => 'no' },
 'niente'	=> { 'x' => 'nothing',
		     '#' => 'non...niente' },
 'no'		=> { 'x' => 'not',
		     '#' => 'no' },
 'non'		=> { 'x' => 'not',
		     '#' => 'adverb?' },
 'nulla'	=> { 'x' => 'nothing',
		     '#' => 'non...nulla' },
 'o'		=> { 'x' => 'or',
		     '#' => 'either',
 		     't' => 'conj' },
 'oggi'		=> { 'x' => 'today',
		     '#' => 'adverb or noun?' },
 'ora'		=> { 'x' => 'now',
		     '#' => 'and n. hour; cf es hora, ahora' },
 'per'		=> { 'x' => 'for',
 		     't' => 'p' },
 'poi'		=> { 'x' => 'then',
		     '#' => 'pues' },
 'poich'	=> { 'x' => 'since',
		     '#' => 'for',
 		     't' => 'conj' },
 'qualche'	=> { 'x' => 'any',
		     '#' => 'some; always followed by singular noun' },
 'qualcuno'	=> { 'x' => 'someone',
		     '#' => 'anyone' },
 'quando'	=> { 'x' => 'when' },
 'questo'	=> { 'x' => 'this' },
 'qui'		=> { 'x' => 'here',
 		     't' => 'adv' },
 'se'		=> { 'x' => 'if',
		     '#' => 'also refl. pron.',
 		     't' => 'conj' },
 'su'		=> { 'x' => 'on',
 		     't' => 'p' },
 'troppo'	=> { 'x' => 'too',
		     '#' => 'demasiado' },
 'tutto'	=> { 'x' => 'all',
		     '#' => 'each, every, everything, everybody' },
 #   Partitive / Contractions
 'al'		=> { 'x' => 'to the',
		     '#' => 'a+il' },
 'ai'		=> { 'x' => 'to the',
		     '#' => 'a+i' },
 'allo'		=> { 'x' => 'to the',
		     '#' => 'a+lo' },
 'agli'		=> { 'x' => 'to the',
		     '#' => 'a+gli' },
 'alla'		=> { 'x' => 'to the',
		     '#' => 'a+la' },
 'alle'		=> { 'x' => 'to the',
		     '#' => 'a+le' },
 'all\''	=> { 'x' => 'to the',
		     '#' => 'a+l\'' },
 'col'		=> { 'x' => 'with the',
		     '#' => 'co+il' },
 'coi'		=> { 'x' => 'with the',
		     '#' => 'co+i' },
 'collo'	=> { 'x' => 'with the',
		     '#' => 'co+lo' },
 'cogli'	=> { 'x' => 'with the',
		     '#' => 'co+gli' },
 'colla'	=> { 'x' => 'with the',
		     '#' => 'co+la' },
 'colle'	=> { 'x' => 'with the',
		     '#' => 'co+le' },
 'coll\''	=> { 'x' => 'with the',
		     '#' => 'co+l\'' },
 'dal'		=> { 'x' => 'by the',
		     '#' => 'da+il' },
 'dai'		=> { 'x' => 'by the',
		     '#' => 'da+i' },
 'dallo'	=> { 'x' => 'by the',
		     '#' => 'da+lo' },
 'dagli'	=> { 'x' => 'by the',
		     '#' => 'da+gli' },
 'dalla'	=> { 'x' => 'by the',
		     '#' => 'da+la' },
 'dalle'	=> { 'x' => 'by the',
		     '#' => 'da+le' },
 'dall\''	=> { 'x' => 'by the',
		     '#' => 'da+l\'' },
 'del'		=> { 'x' => 'of the',
		     '#' => 'di+il' },
 'dei'		=> { 'x' => 'of the',
		     '#' => 'di+i' },
 'dello'	=> { 'x' => 'of the',
		     '#' => 'di+lo' },
 'degli'	=> { 'x' => 'of the',
		     '#' => 'di+gli' },
 'della'	=> { 'x' => 'of the',
		     '#' => 'di+la' },
 'delle'	=> { 'x' => 'of the',
		     '#' => 'di+le' },
 'dell\''	=> { 'x' => 'of the',
		     '#' => 'di+l\'' },
 'nel'		=> { 'x' => 'in the',
		     '#' => 'in+il' },
 'nei'		=> { 'x' => 'in the',
		     '#' => 'in+i' },
 'nello'	=> { 'x' => 'in the',
		     '#' => 'in+lo' },
 'negli'	=> { 'x' => 'in the',
		     '#' => 'in+gli' },
 'nella'	=> { 'x' => 'in the',
		     '#' => 'in+la' },
 'nelle'	=> { 'x' => 'in the',
		     '#' => 'in+le' },
 'nell\''	=> { 'x' => 'in the',
		     '#' => 'in+l\'' },
 'pel'		=> { 'x' => 'for the',
		     '#' => 'per+il' },
 'pei'		=> { 'x' => 'for the',
		     '#' => 'per+i' },
 'sul'		=> { 'x' => 'on the',
		     '#' => 'su+il' },
 'sui'		=> { 'x' => 'on the',
		     '#' => 'su+i' },
 'sullo'	=> { 'x' => 'on the',
		     '#' => 'su+lo' },
 'sugli'	=> { 'x' => 'on the',
		     '#' => 'su+gli' },
 'sulla'	=> { 'x' => 'on the',
		     '#' => 'su+la' },
 'sulle'	=> { 'x' => 'on the',
		     '#' => 'su+le' },
 'sull\''	=> { 'x' => 'on the',
		     '#' => 'su+l\'' },
 # Numbers: cardinal and ordinal
 'primo'	=> { 'x' => 'first' },
 'secondo'	=> { 'x' => 'second' },
 'terzo'	=> { 'x' => 'third' },
 # Days and months
 'domenica'	=> { 'x' => 'sunday' },
 'luned'	=> { 'x' => 'monday' },
 'marted'	=> { 'x' => 'tuesday' },
 'mercoled'	=> { 'x' => 'wednesday' },
 'gioved'	=> { 'x' => 'thursday' },
 'venerd'	=> { 'x' => 'friday' },
 'sabato'	=> { 'x' => 'saturday' },
 'gennaio'	=> { 'x' => 'january' },
 'febbraio'	=> { 'x' => 'february' },
 'marzo'	=> { 'x' => 'march' },
 'aprile'	=> { 'x' => 'april' },
 'maggio'	=> { 'x' => 'may' },
 'giugno'	=> { 'x' => 'june' },
 'luglio'	=> { 'x' => 'july' },
 'agosto'	=> { 'x' => 'august' },
 'settembre'	=> { 'x' => 'september' },
 'ottobre'	=> { 'x' => 'october' },
 'novembre'	=> { 'x' => 'november' },
 'dicembre'	=> { 'x' => 'december' },
 # Key verbs
 #  essere = be
 'essere'	=> { 'x' => 'be',
		     '#' => 'irreg; cf espanol ser',
 		     't' => 'v' },
  'sono'	=> { 'x' => 'are',
		     '#' => 'io/I, loro/they, Loro/you',
 		     'r' => 'essere',
		     't' => 'v' },
  'sei'		=> { 'x' => 'are',
		     '#' => 'tu/you',
 		     'r' => 'essere',
		     't' => 'v' },
  ''		=> { 'x' => 'is',
		     '#' => 'lui, lei, esso, essa, Lei/you',
 		     'r' => 'essere',
		     't' => 'v' },
  'siamo'	=> { 'x' => 'are',
		     '#' => 'noi/we',
 		     'r' => 'essere',
		     't' => 'v' },
  'siete'	=> { 'x' => 'are',
		     '#' => 'voi/you',
 		     'r' => 'essere',
		     't' => 'v' },
 #  essere = be
 'stare'	=> { 'x' => 'be',
		     '#' => 'irreg.; cf espanol estar',
 		     't' => 'v' },
  'sto'		=> { 'x' => 'am',
 		     'r' => 'stare',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's' },
  'stai'	=> { 'x' => 'are',
 		     'r' => 'stare',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's' },
  'sta'		=> { 'x' => 'is',
 		     'r' => 'stare',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's' },
  'stiamo'	=> { 'x' => 'are',
 		     'r' => 'stare',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p' },
  'stano'	=> { 'x' => 'are',
 		     'r' => 'stare',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p' },

 #  avere = have
 'avere'	=> { 'x' => 'have',
 		     't' => 'v' },
  'ho'		=> { 'x' => 'have',
		     '#' => 'io/I',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's' },
  'hai'		=> { 'x' => 'have',
		     '#' => 'tu/you',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's' },
  'ha'		=> { 'x' => 'has',
		     '#' => 'lui, lei, esso, essa, Lei',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's' },
  'abbiamo'	=> { 'x' => 'have',
		     '#' => 'noi/we',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p' },
  'avete'	=> { 'x' => 'have',
		     '#' => 'voi/you',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p' },
  'hanno'	=> { 'x' => 'have',
		     '#' => 'loro/they, Loro/you',
 		     'r' => 'avere',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p' },
 #  dovere - must
 'dovere'	=> { 'x' => 'have to',
		     '#' => 'irreg. be obliged to, must',
 		     't' => 'v' },
 #   virtual base
 'devere'	=> { 'x' => 'have to',
		     '#' => 'irreg. be obliged to, must',
 		     't' => 'v' },
 #  potere - can, to be able to
 'potere'	=> { 'x' => 'can',
		     '#' => 'irreg.',
 		     't' => 'v' },
 #  andare - to go
 'andare'	=> { 'x' => 'go',
 		     't' => 'v' },
  'va'		=> { 'x' => 'goes',
 		     'r' => 'andare',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's' },
 # Vocabulary
 'albero'	=> { 'x' => 'tree',
 		     't' => 'n' },
 'alto'		=> { 'x' => 'tall',
		     '#' => 'high',
 		     't' => 'a' },
 'aprire'	=> { 'x' => 'open',
 		     't' => 'v' },
 'arabo'	=> { 'x' => 'arabic',
		     '#' => 'noun & adj' },
 'armeno'	=> { 'x' => 'armenian',
		     '#' => 'noun & adj' },
 'australia'	=> { 'x' => 'australia',
 		     't' => 'n' },
 'austria'	=> { 'x' => 'austria',
		     '#' => 'proper, place',
		     't' => 'n' },
 'azzurro'	=> { 'x' => 'blue',
 		     't' => 'a' },
 'balcone'	=> { 'x' => 'balcony',
 		     't' => 'n',
		     'g' => 'm' },
 'bambino'	=> { 'x' => 'child',
 		     't' => 'n' },
 'basso'	=> { 'x' => 'low',
 		     't' => 'a' },
 'belgio'	=> { 'x' => 'belgium',
		     '#' => 'proper, place',
		     't' => 'n' },
 'bellino'	=> { 'x' => 'pretty',
 		     't' => 'a' },
 'bello'	=> { 'x' => 'beautiful',
 		     't' => 'a' },
 'bene'		=> { 'x' => 'good',
		     '#' => 'well; adj. and adv.' },
 'bere'		=> { 'x' => 'drink',
 		     't' => 'v' },
 'bianco'	=> { 'x' => 'white',
 		     't' => 'a' },
 'buono'	=> { 'x' => 'good',
 		     't' => 'a' },
 'calzino'	=> { 'x' => 'sock',
 		     't' => 'n' },
 'cantar'	=> { 'x' => 'sing',
		     '#' => '??!',
 		     't' => 'v' },
 'capello'	=> { 'x' => 'hat',
 		     't' => 'n' },
 'capire'	=> { 'x' => 'understand',
 		     't' => 'v' },
 'cartografia'	=> { 'x' => 'cartography',
 		     't' => 'n' },
 'catalano'	=> { 'x' => 'catalan',
		     '#' => 'noun and adj?' },
 'ceco'		=> { 'x' => 'czech',
		     '#' => 'noun and adj' },
 'cercare'	=> { 'x' => 'look for',
		     '#' => 'seek, search',
 		     't' => 'v' },
 'certo'	=> { 'x' => 'certain',
		     '#' => 'sure',
 		     't' => 'a' },
 'cinese'	=> { 'x' => 'chinese',
		     '#' => 'noun & adj' },
 'comodo'	=> { 'x' => 'comfortable',
 		     't' => 'a' },
 'comprare'	=> { 'x' => 'buy',
 		     't' => 'v' },
 'coppia'	=> { 'x' => 'couple',
		     '#' => 'pair',
 		     't' => 'n' },
 'coreano'	=> { 'x' => 'korean',
		     '#' => 'noun & adj' },
 'cosmonauta'	=> { 'x' => 'cosmonaut',
 		     't' => 'n' },
 'cravatta'	=> { 'x' => 'tie',
 		     't' => 'n' },
 'credere'	=> { 'x' => 'believe',
 		     't' => 'v' },
 'cucina'	=> { 'x' => 'kitchen',
 		     't' => 'n' },
 'danese'	=> { 'x' => 'danish',
		     '#' => 'noun & adj' },
 'dizionario'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 'donna'	=> { 'x' => 'woman',
 		     't' => 'n' },
 'dormire'	=> { 'x' => 'sleep',
 		     't' => 'v' },
 'ebraico'	=> { 'x' => 'hebrew',
		     '#' => 'noun & adj?' },
 'edizione'	=> { 'x' => 'edition',
 		     't' => 'n' },
 'elettricit'	=> { 'x' => 'electricity',
 		     't' => 'n' },
 'erba'		=> { 'x' => 'grass',
 		     't' => 'n' },
 'esagerare'	=> { 'x' => 'exxagerate',
 		     't' => 'v' },
 'esperanto'	=> { 'x' => 'esperanto',
 		     't' => 'n' },
 'estro'	=> { 'x' => 'whim',
		     '#' => 'inspiration',
 		     't' => 'n' },
 'facile'	=> { 'x' => 'easy',
 		     't' => 'a' },
 'figlia'	=> { 'x' => 'daughter',
 		     't' => 'n' },
 'finestra'	=> { 'x' => 'window',
 		     't' => 'n' },
 'fiume'	=> { 'x' => 'river',
 		     't' => 'n' },
 'foglio'	=> { 'x' => 'sheet',
		     '#' => 'or leaf of paper',
 		     't' => 'n' },
 'francese'	=> { 'x' => 'french',
		     '#' => 'noun & adj' },
 'francia'	=> { 'x' => 'france',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'fratello'	=> { 'x' => 'brother',
 		     't' => 'n' },
 'gazzetta'	=> { 'x' => 'newspaper',
 		     't' => 'n' },
 'germania'	=> { 'x' => 'germany',
 		     't' => 'n' },
 'gesticolare'	=> { 'x' => 'gesticulate',
 		     't' => 'v' },
 'giappone'	=> { 'x' => 'japan',
 		     't' => 'n' },
 'giapponese'	=> { 'x' => 'japanese',
		     '#' => 'noun & adj' },
 'giardino'	=> { 'x' => 'garden',
 		     't' => 'n' },
 'grande'	=> { 'x' => 'large',
 		     't' => 'a' },
 'greco'	=> { 'x' => 'greek',
		     '#' => 'noun & adj' },
 'guardare'	=> { 'x' => 'look at',
		     '#' => 'examine, regard?',
 		     't' => 'v' },
 'olanda'	=> { 'x' => 'holland',
		     '#' => 'proper, place',
		     't' => 'n' },
 'olandese'	=> { 'x' => 'dutch',
		     '#' => 'noun & adj' },
 'incontro'	=> { 'x' => 'meeting',
 		     't' => 'n' },
 'inghilterra'	=> { 'x' => 'england',
		     '#' => 'proper, place',
		     't' => 'n' },
 'inglese'	=> { 'x' => 'english',
		     '#' => 'noun & adj' },
 'interno'	=> { 'x' => 'inside',
		     '#' => 'interior',
 		     't' => 'n' },
 'islandese'	=> { 'x' => 'icelandic',
		     '#' => 'noun & adj' },
 'lasciare'	=> { 'x' => 'leave',
		     '#' => 'let, allow',
 		     't' => 'v' },
 'latino'	=> { 'x' => 'latin',
		     '#' => 'noun & adj' },
 'leggere'	=> { 'x' => 'read',
 		     't' => 'v' },
 'lettera'	=> { 'x' => 'letter',
 		     't' => 'n' },
 'libro'	=> { 'x' => 'book',
 		     't' => 'n' },
 'linguaggio'	=> { 'x' => 'language',
 		     't' => 'n' },
 'lussemburgo'	=> { 'x' => 'luxembourg',
		     '#' => 'proper, place',
		     't' => 'n' },
 'macchina'	=> { 'x' => 'car',
		     '#' => 'car?, machine?',
 		     't' => 'n' },
 'madre'	=> { 'x' => 'mother',
 		     't' => 'n',
		     'g' => 'f' },
 'mancanza'	=> { 'x' => 'lack',
 		     't' => 'n' },
 'mancare'	=> { 'x' => 'lack',
		     '#' => 'cf es. faltar',
 		     't' => 'v' },
 'mangiare'	=> { 'x' => 'eat',
		     't' => 'v' },
 'mano'		=> { 'x' => 'hand',
 		     't' => 'n',
		     'g' => 'f' },
 'merda'	=> { 'x' => 'shit',
 		     't' => 'n' },
 'messaggio'	=> { 'x' => 'message',
 		     't' => 'n' },
 'minuto'	=> { 'x' => 'minute',
 		     't' => 'n' },
 'modificare'	=> { 'x' => 'modify',
 		     't' => 'v' },
 'momento'	=> { 'x' => 'moment',
 		     't' => 'n' },
 'mondo'	=> { 'x' => 'world',
		     't' => 'n' },
 'muro'		=> { 'x' => 'wall',
 		     't' => 'n' },
 'nome'		=> { 'x' => 'name',
 		     't' => 'n' },
 'norvegia'	=> { 'x' => 'norway',
		     '#' => 'proper, place',
		     't' => 'n' },
 'notare'	=> { 'x' => 'notice',
		     '#' => 'observe',
 		     't' => 'v' },
 'nuotare'	=> { 'x' => 'swim',
 		     't' => 'v' },
 'nuovo'	=> { 'x' => 'new',
 		     't' => 'a' },
 'oltremare'	=> { 'x' => 'overseas',
		     '#' => 'adv.?' },
 'orso'		=> { 'x' => 'bear',
 		     't' => 'n' },
 'ottenere'	=> { 'x' => 'get',
		     '#' => 'obtain; irreg.',
 		     't' => 'v' },
 'padre'	=> { 'x' => 'father',
 		     't' => 'n',
		     'g' => 'm' },
 'pagina'	=> { 'x' => 'page',
 		     't' => 'n' },
 'paraguay'	=> { 'x' => 'paraguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'parola'	=> { 'x' => 'word',
 		     't' => 'n' },
 'partire'	=> { 'x' => 'leave',
		     '#' => 'go away, depart',
		     't' => 'v' },
 'penna'	=> { 'x' => 'pen',
 		     't' => 'n' },
 'piano'	=> { 'x' => 'floor',
 		     't' => 'n' },
 'piccolo'	=> { 'x' => 'small',
 		     't' => 'a' },
 'pigro'	=> { 'x' => 'lazy',
 		     't' => 'a' },
 'portogallo'	=> { 'x' => 'portugal',
		     '#' => 'proper, place',
		     't' => 'n' },
 'posto'	=> { 'x' => 'place',
		     '#' => 'also vers. of v. porre',
 		     't' => 'n' },
 'povero'	=> { 'x' => 'poor',
 		     't' => 'a' },
 'preferire'	=> { 'x' => 'prefer',
 		     't' => 'v' },
 'prestare'	=> { 'x' => 'lend',
		     '#' => 'give',
 		     't' => 'v' },
 'programma'	=> { 'x' => 'program',
 		     't' => 'n' },
 'programmare'	=> { 'x' => 'program',
 		     't' => 'v' },
 'ragazza'	=> { 'x' => 'girl',
 		     't' => 'n' },
 'ragazzo'	=> { 'x' => 'boy',
 		     't' => 'n' },
 'realmente'	=> { 'x' => 'really',
 		     't' => 'adv' },
 'ringraziamento'	=> { 'x' => 'thank',
		     '#' => '?',
 			     't' => 'n' },
 'rosso'	=> { 'x' => 'red',
 		     't' => 'a' },
 'sala'		=> { 'x' => 'room',
 		     't' => 'n' },
 'sapere'	=> { 'x' => 'know',
		     '#' => 'cf. es. saber',
 		     't' => 'v' },
 'sbaglio'	=> { 'x' => 'mistake',
 		     't' => 'n' },
 'scarpa'	=> { 'x' => 'shoe',
 		     't' => 'n' },
 'scrivere'	=> { 'x' => 'write',
 		     't' => 'v' },
 'sedia'	=> { 'x' => 'chair',
 		     't' => 'n' },
 'sguardo'	=> { 'x' => 'glance',
		     '#' => 'look',
 		     't' => 'n' },
 'signora'	=> { 'x' => 'madam',
		     '#' => 'mrs., lady' },
 'signore'	=> { 'x' => 'sir',
		     '#' => 'mister, gentleman' },
 'signorina'	=> { 'x' => 'miss',
		     '#' => 'young lady' },
 'sinistro'	=> { 'x' => 'left',
		     '#' => 'opp. right',
 		     't' => 'a' },
 'solidariet'	=> { 'x' => 'solidarity',
 		     't' => 'n' },
 'sorella'	=> { 'x' => 'sister',
 		     't' => 'n' },
 'spagnuolo'	=> { 'x' => 'spanish',
		     '#' => 'noun and adjective?' },
 'specchio'	=> { 'x' => 'mirror',
 		     't' => 'n' },
 'spillo'	=> { 'x' => 'pin',
 		     't' => 'n' },
 'stanza'	=> { 'x' => 'room',
 		     't' => 'n' },
 'strada'	=> { 'x' => 'street',
 		     't' => 'n' },
 'svezia'	=> { 'x' => 'sweden',
		     '#' => 'proper, place',
		     't' => 'n' },
 'svizzera'	=> { 'x' => 'switzerland',
		     '#' => 'proper, place',
		     't' => 'n' },
 'tailandia'	=> { 'x' => 'thailand',
		     '#' => 'proper, place',
		     't' => 'n' },
 'tasca'	=> { 'x' => 'pocket',
 		     't' => 'n' },
 'tavola'	=> { 'x' => 'table',
 		     't' => 'n' },
 'tedesco'	=> { 'x' => 'german',
		     '#' => 'noun & adj?' },
 'terrazza'	=> { 'x' => 'terrace',
 		     't' => 'n' },
 'tetto'	=> { 'x' => 'roof',
 		     't' => 'n' },
 'tipo'		=> { 'x' => 'guy',
 		     't' => 'n' },
 'traccia'	=> { 'x' => 'trail',
		     't' => 'n' },
 'trovare'	=> { 'x' => 'find',
 		     't' => 'v' },
 'ungherese'	=> { 'x' => 'hungarian',
		     '#' => 'noun & adj' },
 'uomo'		=> { 'x' => 'man',
 		     't' => 'n' },
 'usare'	=> { 'x' => 'use',
 		     't' => 'v' },
 'vendere'	=> { 'x' => 'sell',
 		     't' => 'v' },
 'versione'	=> { 'x' => 'version',
 		     't' => 'n' },
 'vietnam'	=> { 'x' => 'vietnam',
		     '#' => 'proper, place',
		     't' => 'n' },
 'zia'		=> { 'x' => 'aunt',
 		     't' => 'n' },
 'zio'		=> { 'x' => 'uncle',
 		     't' => 'n' },
);
}

1;

