# vi: ts=8 sw=8 sts=8

package Tongues::Magyar;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Central European CP1250
# not ISO 8859-2

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::CentralEuroWin;

$charset = new Charsets::CentralEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun gender
 # Noun plural
 # Adjectives
 # Verb past participle
 # Verb gerund
 # Verb present participle
 # Verb present
 # Verb present subjunctive
 # Verb imperitive
 # Verb preterite
 # Verb imperfect
 # Verb future
 # Verb conditional
 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translatioin		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine		default for words ending in a
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:		verbs
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> case:			nouns, pronouns, adjectives
#	n -> nominative
#	a -> accusative
#	iness -> inessive
#	el -> elative
#	il -> illative
#	sup -> supressive
#	sub -> sublative
#	adess -> adessive
#	ab -> ablative
#	del -> delative
#	al -> allative
#	d -> dative
#	term -> terminative
#	form -> formalis
#	inst -> instrumental
#	trans -> translative
#	ass -> associative
#	caus -> causative

# Hungarian to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 #     Plural
 #    Feminine
 #     Singular
 #     Plural
 #   Indefinite articles
 #  Pronouns & possessive adjectives
 #   Nominative
 'n'		=> { 'x' => 'i',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'n' },
 'te'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'n' },
 'n'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'n' },
 ''		=> { 'x' => 'he',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'n' },
 'mi'		=> { 'x' => 'we',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'n' },
 'ti'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'n' },
 'nk'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'n' },
 'k'		=> { 'x' => 'they',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'n' },
 #   Accusative
 'engem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'a' },
 'tged'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'a' },
 'nt'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'a' },
 't'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'a' },
 'minket'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'a' },
 'titeket'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'a' },
 'nket'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'a' },
 'ket'		=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'a' },
 #   Inessive
 'bennem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'iness' },
 'benned'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'iness' },
 'nben'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'iness' },
 'benne'	=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'iness' },
 'bennnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'iness' },
 'bennetek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'iness' },
 'nkben'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'iness' },
 'bennk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'iness' },
 #   Elative
 'bellem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'el' },
 'belled'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'el' },
 'nbol'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'el' },
 'belle'	=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'el' },
 'bellnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'el' },
 'belletek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'el' },
 'nkbl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'el' },
 'bellk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'el' },
 #   Illative
 'belm'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'il' },
 'beld'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'il' },
 'nbe'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'il' },
 'bel'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'il' },
 'belnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'il' },
 'beltek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'il' },
 'nkbe'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'il' },
 'beljk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'il' },
 #   Supressive
 'rajtam'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'sup' },
 'rajtad'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'sup' },
 'nn'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'sup' },
 'rajta'	=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'sup' },
 'rajtunk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'sup' },
 'rajtatok'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'sup' },
 'nkn'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'sup' },
 'rajtuk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'sup' },
 #   Sublative
 'rm'		=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'sub' },
 'rd'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'sub' },
 'nre'		=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'sub' },
 'r'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'sub' },
 'rnk'		=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'sub' },
 'rtok'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'sub' },
 'nkre'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'sub' },
 'rjuk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'sub' },
 #   Adessive
 'nlam'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'adess' },
 'nlad'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'adess' },
 'nnl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'adess' },
 'nla'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'adess' },
 'nlunk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'adess' },
 'nlatok'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'adess' },
 'nknl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'adess' },
 'nluk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'adess' },
 #   Ablative
 'tolem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'ab' },
 'tled'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'ab' },
 'ntl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'ab' },
 'tle'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'ab' },
 'tlnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'ab' },
 'toletek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'ab' },
 'nktl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'ab' },
 'tlk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'ab' },
 #   Delative
 'rlam'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'del' },
 'rlad'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'del' },
 'nrl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'del' },
 'rla'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'del' },
 'rlunk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'del' },
 'rlatok'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'del' },
 'nkrl'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'del' },
 'rluk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'del' },
 #   Allative
 'hozzm'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'al' },
 'hozzd'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'al' },
 'nhz'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'al' },
 'hozz'	=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'al' },
 'hozznk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'al' },
 'hozztok'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'al' },
 'nkhz'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'al' },
 'hozzjuk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'al' },
 #   Dative
 'nekem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'd' },
 'neked'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'd' },
 'nnek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'd' },
 'neki'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'd' },
 'neknk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'd' },
 'nektek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'd' },
 'nknek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'd' },
 'nekik'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'd' },
 #   Associative
 'velem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'ass' },
 'veled'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'ass' },
 'nnel'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'ass' },
 'vele'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'ass' },
 'velnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'ass' },
 'veletek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'ass' },
 'nkkel'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'ass' },
 'velk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'ass' },
 #   Causative
 'rtem'	=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'caus' },
 'rted'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'caus' },
 'nrt'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 's',
		     'c' => 'caus' },
 'rte'		=> { 'x' => 'him',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'caus' },
 'rtnk'	=> { 'x' => 'us',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'caus' },
 'rtetek'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'caus' },
 'nkrt'	=> { 'x' => 'you',
		     't' => 'pro',
		     'p' => '2',
		     '#' => 'formal',
		     'n' => 'p',
		     'c' => 'caus' },
 'rtk'	=> { 'x' => 'them',
		     't' => 'pro',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'caus' },
 #  Other functional words
 #   Partitive / Contractions
 # Numbers: cardinal and ordinal
 # Days and months
 'vasrnap'	=> { 'x' => 'sunday' },
 'htf'	=> { 'x' => 'monday' },
 'kedd'		=> { 'x' => 'tuesday' },
 'szerda'	=> { 'x' => 'wednesday' },
 'cstrtk'	=> { 'x' => 'thursday' },
 'pntek'	=> { 'x' => 'friday' },
 'szombat'	=> { 'x' => 'saturday' },
 'janur'	=> { 'x' => 'january' },
 'februr'	=> { 'x' => 'february' },
 'mrcius'	=> { 'x' => 'march' },
 'prilis'	=> { 'x' => 'april' },
 'mjus'	=> { 'x' => 'may' },
 'jnius'	=> { 'x' => 'june' },
 'jlius'	=> { 'x' => 'july' },
 'augusztus'	=> { 'x' => 'august' },
 'szeptember'	=> { 'x' => 'september' },
 'oktber'	=> { 'x' => 'october' },
 'november'	=> { 'x' => 'november' },
 'december'	=> { 'x' => 'december' },
 # Key verbs
 # Vocabulary
 'ausztrlia'	=> { 'x' => 'australia',
 		     't' => 'n' },
 'cikk'		=> { 'x' => 'article',
 		     't' => 'n' },
 'knyv'	=> { 'x' => 'book',
 		     't' => 'n' },
 'sztr'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 'trkpszet'	=> { 'x' => 'cartography',
 		     't' => 'n' },
 'rhajs'	=> { 'x' => 'astronaut',
 		     't' => 'n' },
);
}

1;

