# vi: ts=8 sw=8

package Tongues::Slovencine;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Central European code page 1250
# not ISO 8859-2

# Alphabetical order
# A B C  D E F G H CH I J K L M N O P Q R  S  T U V W X Y Z 
# a b c  d e f g h ch i j k l m n o p q r  s  t u v w x y z 
# Variant letters
#           
#           

# Soft consonants: c   j    

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::CentralEuroWin;

$charset = new Charsets::CentralEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun case
 # Noun gender
 # Noun plural
 # Adjectives
 # Verb past participle
 # Verb gerund
 # Verb present participle
 # Verb present
 # Verb present subjunctive
 # Verb imperitive
 # Verb preterite
 # Verb imperfect
 # Verb future
 # Verb conditional
 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translatioin		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	d -> dual
#	t -> trial
#	p -> plural
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> case:
#	n -> nominative 		default
#	a -> accusative 

# Slovencine to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 #     Plural
 #    Feminine
 #     Singular
 #     Plural
 #   Indefinite articles
 #  Pronouns & possessive adjectives
 #   1st person
 #   2nd person
 #   3rd person
 #   Other pronouns
 #  Interrogatives
 #  Other functional words
 'de'	=> { 'x' => 'of',
	     't' => 'p' },
 #   Partitive / Contractions
 # Numbers: cardinal and ordinal
 # Days and months
 'nedee'	=> { 'x' => 'sunday' },
 'pondelok'	=> { 'x' => 'monday' },
 'utorok'	=> { 'x' => 'tuesday' },
 'streda'	=> { 'x' => 'wednesday' },
 'tvrtok'	=> { 'x' => 'thursday' },
 'piatok'	=> { 'x' => 'friday' },
 'sobota'	=> { 'x' => 'saturday' },
 'janur'	=> { 'x' => 'january' },
 'februr'	=> { 'x' => 'february' },
 'marec'	=> { 'x' => 'march' },
 'aprl'	=> { 'x' => 'april' },
 'mj'		=> { 'x' => 'may' },
 'jn'		=> { 'x' => 'june' },
 'jl'		=> { 'x' => 'july' },
 'august'	=> { 'x' => 'august' },
 'september'	=> { 'x' => 'september' },
 'oktber'	=> { 'x' => 'october' },
 'november'	=> { 'x' => 'november' },
 'december'	=> { 'x' => 'december' },
 # Key verbs
 # Vocabulary
 'lnok'	=> { 'x' => 'article',
		     't' => 'n' },
);
}

1;

