#! /usr/bin/perl
# vi: ts=4 sw=4 sts=4

require 5.002;
require Linguaphile;

use strict;

use lib ".";

my $default_src_lang = $ARGV[1];
my $default_dst_lang = $ARGV[2];

use Getopt::Std;

use vars qw($opt_s $opt_d $opt_a $opt_l $opt_q $opt_P $opt_U);

my $line;

getopts('s:d:alqPU');

$opt_s = $default_src_lang unless $opt_s;
$opt_d = $default_dst_lang unless $opt_d;

my $fh = *STDIN;

# Filename on command line?
if ($ARGV[0]) {
	open $fh, $ARGV[0] or die "** File not found";
}

# Main loop

my $translator = new Linguaphile;

$translator->init($opt_s, $opt_d);
$opt_a && $translator->set_untrans_brackets( "\e[01;33m", "\e[0m" );

$opt_q || print STDERR "** translating from $translator->{src_eng_name} to $translator->{dst_eng_name}:\n\n";

while ($line = <$fh>) {
	# TODO Should support arbitrary encodings
	$line =~ tr/\x80-\xff/\xc4\xc5\xc7\xc9\xd1\xd6\xdc\xe1\xe0\xe2\xe4\xe3\xe5\xe7\xe9\xe8\xea\xeb\xed\xec\xee\xef\xf1\xf3\xf2\xf4\xf6\xf5\xfa\xf9\xfb\xfc\x86\xb0\xa2\xa3\xa7\x95\xb6\xdf\xae\xa9\x99\xb4\xa8?\xc6\xd8?\xb1??\xa5\xb5?????\xaa\xba?\xe6\xf8\xbf\xa1\xac?\x83??\xab\xbb\x85\xa0\xc0\xc3\xd5\x8c\x9c\x96\x97\x93\x94\x91\x92\xf7?\xff\x9f?\x80\x8b\x9b??\x87\xb7\x82\x84\x89\xc2\xca\xc1\xcb\xc8\xcd\xce\xcf\xcc\xd3\xd4?\xd2\xda\xdb\xd9?\x88\x98\xaf???\xb8???/ if ($^O eq 'MacOS');

	if ($opt_P) {
		$opt_a && print "\e[01;35m";	# ansi magenta
		print $line;
		$opt_a && print "\e[0m";		# ansi normal
	}

	my $tr_line = $translator->translate($line, $opt_U ? 1 : 0);

	# TODO Should support arbitrary encodings
	$tr_line =~ tr/\x80-\xff/\xdb?\xe2\xc4\xe3\xc9\xa0\xe0\xf6\xe4?\xdc\xce????\xd4\xd5\xd2\xd3\xa5\xd0\xd1\xf7\xaa?\xdd\xcf??\xd9\xca\xc1\xa2\xa3?\xb4?\xa4\xac\xa9\xbb\xc7\xc2?\xa8\xf8\xa1\xb1??\xab\xb5\xa6\xe1\xfc?\xbc\xc8???\xc0\xcb\xe7\xe5\xcc\x80\x81\xae\x82\xe9\x83\xe6\xe8\xed\xea\xeb\xec?\x84\xf1\xee\xef\xcd\x85?\xaf\xf4\xf2\xf3\x86??\xa7\x88\x87\x89\x8b\x8a\x8c\xbe\x8d\x8f\x8e\x90\x91\x93\x92\x94\x95?\x96\x98\x97\x99\x9b\x9a\xd6\xbf\x9d\x9c\x9e\x9f??\xd8/ if ($^O eq 'MacOS');

	print "$tr_line";
}

exit;

